﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using Syncfusion.Data.Extensions;

namespace DataGridDemo
{
    public class CustomTypeDescriptorInfo : CustomTypeDescriptorBase, INotifyPropertyChanged 
    {
        private readonly Dictionary<string, CellData> _data = new Dictionary<string, CellData>();
        private int? _orderID;
        private CellData _employeeID;

        public CustomTypeDescriptorInfo()
        {

        }

        public void SetData(string key, CellData value)
        {
            if(key.ContainsSpecialChars())//we will use these as property names
                throw new ArgumentException("key cannot illegal chars");

            if (_data.ContainsKey(key))
                _data[key] = value;
            else
                _data.Add(key, value);

            RaisePropertyChanged(key);
        }

        public IEnumerable<string> GetKeys()
        {
            return _data.Keys;
        }

        public CellData GetValue(string key)
        {
            return _data.ContainsKey(key) ? _data[key] : null;
        }


        #region Public Properties

        public int? OrderID
        {
            get
            {
                return _orderID;
            }
            set
            {
                this._orderID = value;
                RaisePropertyChanged("OrderID");
            }
        }

        public CellData EmployeeID
        {
            get
            {
                return _employeeID;
            }
            set
            {
                this._employeeID = value;
                RaisePropertyChanged("EmployeeID");
            }
        }
        
        #endregion

        #region INotifyPropertyChanged implementation

        
        public event PropertyChangedEventHandler PropertyChanged;

        private void RaisePropertyChanged(string propertyName)
        {
            if(this.PropertyChanged != null)
                this.PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        ///TODO This was modified To show dynamic Props based on what is added to the dictionary this is what the grid will not bind to
        #region ICustomTypeDecriptorInterface

        public override PropertyDescriptorCollection GetProperties()
        {
            var nativeProps = base.GetProperties();
            List<PropertyDescriptor> fullList = new List<PropertyDescriptor>();

            foreach (PropertyDescriptor prop in nativeProps)
                fullList.Add(prop);

            foreach (var pair in _data)
                fullList.Add(new DynamicPropertyDescriptor(pair.Key));

            return new PropertyDescriptorCollection(fullList.ToArray());
        }

        public override PropertyDescriptorCollection GetProperties(Attribute[] attributes)
        {
            var nativeProps = base.GetProperties(attributes);
            List<PropertyDescriptor> fullList = new List<PropertyDescriptor>();

            foreach (PropertyDescriptor prop in nativeProps)
                fullList.Add(prop);

            foreach (var pair in _data)
                fullList.Add(new DynamicPropertyDescriptor(pair.Key));

            return new PropertyDescriptorCollection(fullList.ToArray());
        }


        #endregion
    }
}

